module ReorderTools

import ..Ahorn
using Gtk, Gtk.ShortNames, Gtk.GConstants

# Allow dragging tools to reorder them
GAccessor.reorderable(Ahorn.toolList.tree, true)

# Rest of the file: Enable persistence for the tool ordering
# some code copied from tools.jl
# Note tools are not loaded yet so we can only update the list later
# Monkey-patching Ahorn functions doesn't work because of julia world age

# Load the tool order
toolOrder = get(Ahorn.persistence, "tool_order", nothing)

# Update & save the new tool order when it changes
signal_connect(Ahorn.toolList.tree, "drag-end") do widget, event
    global toolOrder = [Ahorn.toolList.store[i][1] for i = 1:length(Ahorn.toolList.store)]
    Ahorn.persistence["tool_order"] = copy(toolOrder)
end

# sort the list of tool names with the new order
function newSortedToolNames(tools::Array{String, 1})
    global toolOrder
    if toolOrder == nothing
        toolOrder = Ahorn.getSortedToolNames(tools)
    else
        # Update tool order based on loaded tools
        loadedToolNames = Ahorn.getSortedToolNames(tools)
        # Remove tools that were not loaded
        newToolOrder = filter(name -> name in loadedToolNames, toolOrder)
        # New tool (eg added by debug hotswapping) goes at end
        for toolname in loadedToolNames
            if !(toolname in newToolOrder)
                push!(newToolOrder, toolname)
            end
        end
        toolOrder = newToolOrder
    end
    return toolOrder
end

# Apply the new tool order to the tool list
function updateToolList!(list::Ahorn.ListContainer)
    if haskey(Ahorn.persistence, "selected_tool")
        toolName = Ahorn.persistence["selected_tool"]
        toolDisplayName = Ahorn.getToolName(Ahorn.persistence["selected_tool"])
        Ahorn.updateTreeView!(list, newSortedToolNames(Ahorn.loadedTools), row -> row[1] == toolName || row[1] == toolDisplayName)
    else
        Ahorn.updateTreeView!(list, newSortedToolNames(Ahorn.loadedTools), 1)
    end
end

# Update tool change hotkeys to use the correct sorting
# these hotkeys were already set in tools.jl
function toolChangeCallback(index)
    sortedToolNames = newSortedToolNames(Ahorn.loadedTools)
    if length(sortedToolNames) >= index
        Ahorn.changeTool!(sortedToolNames[index])
        Ahorn.selectRow!(Ahorn.toolList, index)
        return true
    end
    return false
end
for i in 1:length(Ahorn.toolChangeHotkeys)
    hotkey = Ahorn.toolChangeHotkeys[i]
    Ahorn.toolChangeHotkeys[i] = Ahorn.Hotkey(hotkey.key, () -> toolChangeCallback(i), hotkey.conditions)
end

# Re-order tool list when window is ready
rememberToolListWaiter = signal_connect(Ahorn.window, "window-state-event") do window, event
    signal_handler_disconnect(window, rememberToolListWaiter)
    updateToolList!(Ahorn.toolList)
    return false
end

end
